;
; MSTimer - Microsecond resolution timer for BASIC.
;
; Based on C/ASM routines (C)1991 by Bruce Ackerman of
; Convivia software, who I would gladly have contacted for 
; permission had he left an address or number; however,
; since he released it with full source and no documentation,
; I've taken the liberty to assume he wouldn't mind as long
; as credit was given.
;
; Adapted for BASIC use with Microsoft QuickAssembler v2.51
; by Brent Ashley (416)314-4363 
;
; These routines use your 8253 timer chip to generate an
; interrupt at the rate of 1.19318 MHz.  The count returned
; by the MSTimerRead function is a long integer and will
; increase by 1,193,180 each second.
;
; use the following declares in your BASIC code:
;
; DECLARE SUB MSTimerInstall()
; DECLARE SUB MSTimerZero()
; DECLARE SUB MSTimerClose()
; DECLARE FUNCTION MSTimerRead&()
;
; Remember to use MSTimerClose to avoid crashes due to 
; misdirected interrupts.
;

.model medium, basic
.code

oldint1c dd 0
mscount dw 0

MSTimerInstall proc 

     ; Install timer into int 1c

     ; use timer chip channel 0 in mode 2
     mov al, 034h 
     out 43h, al
     xor al, al
     out 40h, al
     out 40h, al
     
     ; store old interrupt 1c vector
     mov ax, 351ch 
     int 21h
     mov word ptr cs:oldint1c, bx
     mov word ptr cs:oldint1c + 2, es

     ; install our handler into vector table
     mov dx, offset ourint1c
     push ds
     push cs
     pop ds
     mov ax,251ch
     int 21h
     pop ds
     
     ; skip around resident code
     jmp retrn1

ourint1c:
     inc cs:mscount     ; handler 2 chains to original interrupt
     jmp dword ptr cs:oldint1c

retrn1:
     ret
    
MSTimerInstall endp

MSTimerZero proc

     ; clear timer count

     xor ax,ax
     mov cs:mscount, ax
     mov cs:mscount + 2, ax
     
     ret
     
MSTimerZero endp     

MSTimerRead proc 

     ; read timer count - returns count based on
     ; 1.19318 MHz clock - NOT MICROSECONDS
     
     mov bx, cs:mscount 
     mov al, 06h
     out 43h, al
     in al, 40h
     
     ; swap bytes and twos-complement for upcount
     xchg al,ah         
     not ax
     inc ax
     
     ; check if count changed during read process
     mov dx, cs:mscount 
     cmp bx, dx
     je retrn2
     cmp ax, 8000h
     jb retrn2
     mov dx,bx
     
retrn2:
     ; dx:ax contains count to return as long to BASIC
     ret

MSTimerRead endp

MSTimerClose proc 

     ; clean up and uninstall hooked vector

     ; set timer chip back to mode 3 and clear count
     mov al,036h  
     out 43h, al
     xor al,al
     out 40h, al  
     out 40h, al
     
     ; unload if we were loaded
     push ds
     lds dx, cs:oldint1c 
     test dx,dx
     jz notloaded
     mov ax, 251ch
     int 21h                   ; reset vector

notloaded:
     pop ds

     ret
     
MSTimerClose endp

     end

; [end of file]

